"""Copyright (c) 2024 Nathaniel Starkman. All rights reserved.

oncequinox: a module for creating singleton Equinox modules.

"""

__all__ = ("SingletonModuleMeta",)

from ._singleton import SingletonModuleMeta
